@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006, KATO Noriaki
+ * Copyright (c) 2006-2009, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,8 +27,8 @@
  * \brief set/multisetコンテナ
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-11-11
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.4.0/cstl/set.h $
- * $Id: set.h 181 2007-12-11 00:48:23Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.1/cstl/set.h $
+ * $Id: set.h 264 2009-04-17 02:29:36Z katono $
  */
 #ifndef CSTL_SET_H_INCLUDED
 #define CSTL_SET_H_INCLUDED
@@ -47,31 +47,39 @@
 
 
 #define CSTL_COMMON_SET_IMPLEMENT(Name, Type, Compare)	\
-/*! 
- * \brief setノード構造体
+/*! \
+ * \brief set赤黒木構造体\
  */\
-struct Name##RBTreeNode_t {\
-	struct Name##RBTreeNode_t *parent;\
-	struct Name##RBTreeNode_t *left;\
-	struct Name##RBTreeNode_t *right;\
+struct Name##RBTree {\
+	struct Name##RBTree *parent;\
+	struct Name##RBTree *left;\
+	struct Name##RBTree *right;\
 	int color;\
 	Type key;\
-	CSTL_RBTREE_MAGIC(struct Name##RBTreeNode_t *magic;)\
+	CSTL_RBTREE_MAGIC(struct Name##RBTree *magic;)\
 };\
 \
 CSTL_RBTREE_WRAPPER_IMPLEMENT(Name, Type, Type, Compare)\
 \
-static Name##RBTreeNode *Name##RBTreeNode_new(Type elem, int color)\
+static Name##RBTree *Name##RBTree_new_node(Type const *data, int color)\
 {\
-	Name##RBTreeNode *self;\
-	self = (Name##RBTreeNode *) malloc(sizeof(Name##RBTreeNode));\
-	if (!self) return 0;\
-	self->key = elem;\
-	self->left = (Name##RBTreeNode *) &Name##RBTree_nil;\
-	self->right = (Name##RBTreeNode *) &Name##RBTree_nil;\
-	self->parent = (Name##RBTreeNode *) &Name##RBTree_nil;\
-	self->color = color;\
-	return self;\
+	Name##RBTree *node;\
+	node = (Name##RBTree *) malloc(sizeof(Name##RBTree));\
+	if (!node) return 0;\
+	node->key = *data;\
+	node->left = (Name##RBTree *) &Name##RBTree_nil;\
+	node->right = (Name##RBTree *) &Name##RBTree_nil;\
+	node->parent = (Name##RBTree *) &Name##RBTree_nil;\
+	node->color = color;\
+	return node;\
+}\
+\
+Type const *Name##_data(Name##Iterator pos)\
+{\
+	assert(pos && "Set_data");\
+	assert(pos->magic && "Set_data");\
+	assert(!CSTL_RBTREE_IS_HEAD(pos) && "Set_data");\
+	return &pos->key;\
 }\
 \
 
@@ -86,7 +94,8 @@ static Name##RBTreeNode *Name##RBTreeNode_new(Type elem, int color)\
 \
 CSTL_SET_BEGIN_EXTERN_C()\
 CSTL_RBTREE_WRAPPER_INTERFACE(Name, Type, Type)\
-Name##Iterator Name##_insert(Name *self, Type elem, int *success);\
+Name##Iterator Name##_insert(Name *self, Type data, int *success);\
+Type const *Name##_data(Name##Iterator pos);\
 CSTL_SET_END_EXTERN_C()\
 
 /*! 
@@ -99,18 +108,18 @@ CSTL_SET_END_EXTERN_C()\
 #define CSTL_SET_IMPLEMENT(Name, Type, Compare)	\
 CSTL_COMMON_SET_IMPLEMENT(Name, Type, Compare)\
 \
-Name##Iterator Name##_insert(Name *self, Type elem, int *success)\
+Name##Iterator Name##_insert(Name *self, Type data, int *success)\
 {\
 	Name##Iterator pos;\
 	assert(self && "Set_insert");\
 	assert(self->magic == self && "Set_insert");\
-	pos = Name##RBTree_find(self->tree, elem);\
+	pos = Name##RBTree_find(self->tree, data);\
 	if (pos == Name##RBTree_end(self->tree)) {\
-		pos = Name##RBTreeNode_new(elem, CSTL_RBTREE_RED);\
+		pos = Name##RBTree_new_node(&data, CSTL_RBTREE_RED);\
 		if (pos) {\
 			Name##RBTree_insert(self->tree, pos);\
 			if (success) *success = 1;\
-			self->nelems++;\
+			self->size++;\
 		} else {\
 			if (success) *success = 0;\
 		}\
@@ -124,7 +133,7 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 {\
 	register Name##Iterator pos;\
 	register Name##Iterator tmp;\
-	Name##RBTreeNode head;\
+	Name##RBTree head;\
 	register size_t count = 0;\
 	assert(self && "Set_insert_range");\
 	assert(self->magic == self && "Set_insert_range");\
@@ -132,11 +141,11 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 	assert(last && "Set_insert_range");\
 	assert(first->magic && "Set_insert_range");\
 	assert(last->magic && "Set_insert_range");\
-	head.right = (Name##RBTreeNode *) &Name##RBTree_nil;\
+	head.right = (Name##RBTree *) &Name##RBTree_nil;\
 	tmp = &head;\
 	for (pos = first; pos != last; pos = Name##_next(pos)) {\
-		if (Name##RBTree_find(self->tree, Name##_key(pos)) == Name##RBTree_end(self->tree)) {\
-			tmp->right = Name##RBTreeNode_new(Name##_key(pos), CSTL_RBTREE_RED);\
+		if (Name##RBTree_find(self->tree, *Name##_data(pos)) == Name##RBTree_end(self->tree)) {\
+			tmp->right = Name##RBTree_new_node(Name##_data(pos), CSTL_RBTREE_RED);\
 			if (!tmp->right) {\
 				for (pos = head.right; pos != 0; pos = tmp) {\
 					tmp = pos->right;\
@@ -148,12 +157,12 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 			count++;\
 		}\
 	}\
-	for (pos = head.right; pos != (Name##RBTreeNode *) &Name##RBTree_nil; pos = tmp) {\
+	for (pos = head.right; pos != (Name##RBTree *) &Name##RBTree_nil; pos = tmp) {\
 		tmp = pos->right;\
-		pos->right = (Name##RBTreeNode *) &Name##RBTree_nil;\
+		pos->right = (Name##RBTree *) &Name##RBTree_nil;\
 		Name##RBTree_insert(self->tree, pos);\
 	}\
-	self->nelems += count;\
+	self->size += count;\
 	return 1;\
 }\
 \
@@ -169,7 +178,8 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 \
 CSTL_SET_BEGIN_EXTERN_C()\
 CSTL_RBTREE_WRAPPER_INTERFACE(Name, Type, Type)\
-Name##Iterator Name##_insert(Name *self, Type elem);\
+Name##Iterator Name##_insert(Name *self, Type data);\
+Type const *Name##_data(Name##Iterator pos);\
 CSTL_SET_END_EXTERN_C()\
 
 /*! 
@@ -182,15 +192,15 @@ CSTL_SET_END_EXTERN_C()\
 #define CSTL_MULTISET_IMPLEMENT(Name, Type, Compare)	\
 CSTL_COMMON_SET_IMPLEMENT(Name, Type, Compare)\
 \
-Name##Iterator Name##_insert(Name *self, Type elem)\
+Name##Iterator Name##_insert(Name *self, Type data)\
 {\
 	Name##Iterator pos;\
 	assert(self && "MultiSet_insert");\
 	assert(self->magic == self && "MultiSet_insert");\
-	pos = Name##RBTreeNode_new(elem, CSTL_RBTREE_RED);\
+	pos = Name##RBTree_new_node(&data, CSTL_RBTREE_RED);\
 	if (pos) {\
 		Name##RBTree_insert(self->tree, pos);\
-		self->nelems++;\
+		self->size++;\
 	}\
 	return pos;\
 }\
@@ -199,7 +209,7 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 {\
 	register Name##Iterator pos;\
 	register Name##Iterator tmp;\
-	Name##RBTreeNode head;\
+	Name##RBTree head;\
 	register size_t count = 0;\
 	assert(self && "MultiSet_insert_range");\
 	assert(self->magic == self && "MultiSet_insert_range");\
@@ -207,10 +217,10 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 	assert(last && "MultiSet_insert_range");\
 	assert(first->magic && "MultiSet_insert_range");\
 	assert(last->magic && "MultiSet_insert_range");\
-	head.right = (Name##RBTreeNode *) &Name##RBTree_nil;\
+	head.right = (Name##RBTree *) &Name##RBTree_nil;\
 	tmp = &head;\
 	for (pos = first; pos != last; pos = Name##_next(pos)) {\
-		tmp->right = Name##RBTreeNode_new(Name##_key(pos), CSTL_RBTREE_RED);\
+		tmp->right = Name##RBTree_new_node(Name##_data(pos), CSTL_RBTREE_RED);\
 		if (!tmp->right) {\
 			for (pos = head.right; pos != 0; pos = tmp) {\
 				tmp = pos->right;\
@@ -221,12 +231,12 @@ int Name##_insert_range(Name *self, Name##Iterator first, Name##Iterator last)\
 		tmp = tmp->right;\
 		count++;\
 	}\
-	for (pos = head.right; pos != (Name##RBTreeNode *) &Name##RBTree_nil; pos = tmp) {\
+	for (pos = head.right; pos != (Name##RBTree *) &Name##RBTree_nil; pos = tmp) {\
 		tmp = pos->right;\
-		pos->right = (Name##RBTreeNode *) &Name##RBTree_nil;\
+		pos->right = (Name##RBTree *) &Name##RBTree_nil;\
 		Name##RBTree_insert(self->tree, pos);\
 	}\
-	self->nelems += count;\
+	self->size += count;\
 	return 1;\
 }\
 \
